@@ -6,13 +6,13 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.29';
+our $VERSION = '0.46';
 
 use File::Copy;
 use File::Path;
+use File::Which;
 use Log::Log4perl qw(:easy);
 use Log::Log4perl::Util;
-use LWP::Simple;
 use File::Basename;
 use File::Spec::Functions qw(rel2abs abs2rel);
 use Cwd;
@@ -76,7 +76,7 @@ our @EXPORTABLE = qw(
 cp rmf mkd cd make 
 cdback download untar 
 pie slurp blurt mv tap 
-plough qquote quote perm_cp
+plough qquote quote perm_cp owner_cp
 perm_get perm_set
 sysrun untar_in pick ask
 hammer say
@@ -84,6 +84,8 @@ sudo_me bin_find
 fs_read_open fs_write_open pipe_copy
 snip password_read nice_time
 def_or blurt_atomic
+is_utf8_data utf8_available
+printable home_dir
 );
 
 our %EXPORTABLE = map { $_ => 1 } @EXPORTABLE;
@@ -147,13 +149,13 @@ Sysadm::Install - Typical installation tasks for system administrators
 =head1 DESCRIPTION
 
 Have you ever wished for your installation shell scripts to run
-reproducably, without much programming fuzz, and even with optional
+reproducibly, without much programming fuzz, and even with optional
 logging enabled? Then give up shell programming, use Perl.
 
 C<Sysadm::Install> executes shell-like commands performing typical
 installation tasks: Copying files, extracting tarballs, calling C<make>.
 It has a C<fail once and die> policy, meticulously checking the result
-of every operation and calling C<die()> immeditatly if anything fails.
+of every operation and calling C<die()> immediately if anything fails.
 
 C<Sysadm::Install> also supports a I<dry_run> mode, in which it 
 logs everything, but suppresses any write actions. Dry run mode
@@ -175,10 +177,6 @@ program's point of view.
 
 =over 4
 
-=cut
-
-=pod
-
 =item C<cp($source, $target)>
 
 Copy a file from C<$source> to C<$target>. C<target> can be a directory.
@@ -192,7 +190,8 @@ shown below.
 sub cp {
 ###############################################
 
-    local($Log::Log4perl::caller_depth) += 1;
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     _confirm("cp $_[0] $_[1]") or return 1;
 
@@ -214,7 +213,8 @@ Move a file from C<$source> to C<$target>. C<target> can be a directory.
 sub mv {
 ###############################################
 
-    local($Log::Log4perl::caller_depth) += 1;
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     _confirm("mv $_[0] $_[1]") or return 1;
 
@@ -238,16 +238,26 @@ sub download {
 ###############################################
     my($url) = @_;
 
-    local($Log::Log4perl::caller_depth) += 1;
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     INFO "download $url";
 
     _confirm("Downloading $url => ", basename($url)) or return 1;
 
-    my $rc = getstore($url, basename($_[0]));
+    require LWP::UserAgent;
+    require HTTP::Request;
+    require HTTP::Status;
+
+    my $ua = LWP::UserAgent->new();
+    my $request = HTTP::Request->new(GET => $url);
+    my $response = $ua->request($request, basename($_[0]));
+    my $rc = $response->code();
     
-    if($rc != RC_OK) {
-        LOGCROAK("Cannot download $_[0] ($!)");
+    if($rc != HTTP::Status::RC_OK()) {
+        LOGCROAK("Cannot download $_[0] (", 
+                  $response->message(),
+                 ")");
     }
 
     return 1;
@@ -274,7 +284,9 @@ IO::Zlib installed.
 ###############################################
 sub untar {
 ###############################################
-    local($Log::Log4perl::caller_depth) += 1;
+
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     LOGCROAK("untar called without defined tarfile") unless 
          @_ == 1 and defined $_[0];
@@ -287,29 +299,31 @@ sub untar {
     require Archive::Tar;
     my $arch = Archive::Tar->new($_[0]);
 
+    my @extracted = ();
+
     if($nice and $topdir eq $namedir) {
         DEBUG "Nice archive, extracting to subdir $topdir";
-        rmf($namedir);
-        $arch->extract();
+        @extracted = $arch->extract();
     } elsif($nice) {
         DEBUG "Not-so-nice archive topdir=$topdir namedir=$namedir";
-        rmf($namedir);
-        rmf($topdir);
             # extract as topdir
-        $arch->extract();
+        @extracted = $arch->extract();
         rename $topdir, $namedir or 
             LOGCROAK("Can't rename $topdir, $namedir");
     } else {
         LOGCROAK("no topdir") unless defined $topdir;
         DEBUG "Not-so-nice archive (no topdir), extracting to subdir $topdir";
         $topdir = basename $topdir;
-        rmf($topdir);
         mkd($topdir);
         cd($topdir);
-        $arch->extract();
+        @extracted = $arch->extract();
         cdback();
     }
 
+    if( !@extracted ) {
+        LOGCROAK "Archive $_[0] was empty.";
+    }
+
     return $topdir;
 }
 
@@ -327,7 +341,8 @@ sub untar_in {
 ###############################################
     my($tar_file, $dir) = @_;
 
-    local($Log::Log4perl::caller_depth) += 1;
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     LOGCROAK("not enough arguments") if
       ! defined $tar_file or ! defined $dir;
@@ -350,7 +365,7 @@ sub untar_in {
 
 =pod
 
-=item C<pick($prompt, $options, $default)>
+=item C<pick($prompt, $options, $default, $opts)>
 
 Ask the user to pick an item from a displayed list. C<$prompt>
 is the text displayed, C<$options> is a referenc to an array of
@@ -374,35 +389,39 @@ If the user enters C<1>, C<2> or C<3>, the corresponding text string
 (C<"apple">, C<"pear">, C<"pineapple"> will be returned by
 C<pick()>.
 
+If the optional C<$opts> hash has C<{ tty =E<gt> 1 }> set, then 
+the user reponse will be expected from the console, not STDIN.
+
 =cut
 
 ##################################################
 sub pick {
 ##################################################
-    my ($prompt, $options, $default) = @_;    
+    my ($prompt, $options, $default, $opts) = @_;    
 
-    local($Log::Log4perl::caller_depth) += 1;
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     my $default_int;
     my %files;
 
-    if(@_ != 3 or ref($options) ne "ARRAY") {
+    if(@_ < 3 or ref($options) ne "ARRAY") {
         LOGCROAK("pick called with wrong #/type of args");
     }
     
     {
         my $count = 0;
 
+        my $user_prompt = "";
+
         foreach (@$options) {
-            print STDERR "[", ++$count, "] $_\n";
+            $user_prompt .= "[" . ++$count . "] $_\n";
             $default_int = $count if $count eq $default;
             $files{$count} = $_;
         }
     
-        print STDERR "$prompt [$default_int]> "
-            or die "Couldn't write STDERR: ($!)";
-        my $input = <STDIN>;
-        chomp($input) if defined $input;
+        $user_prompt .= "$prompt [$default_int]> ";
+        my $input = user_input($user_prompt, $opts);
 
         $input = $default_int if !defined $input or !length($input);
 
@@ -415,35 +434,58 @@ sub pick {
 
 =pod
 
-=item C<ask($prompt, $default)>
+=item C<ask($prompt, $default, $opts)>
 
 Ask the user to either hit I<Enter> and select the displayed default
 or to type in another string.
 
+If the optional C<$opts> hash has C<{ tty =E<gt> 1 }> set, then 
+the user reponse will be expected from the console, not STDIN.
+
 =cut
 
 ##################################################
 sub ask {
 ##################################################
-    my ($prompt, $default) = @_;    
+    my ($prompt, $default, $opts) = @_;    
 
-    local($Log::Log4perl::caller_depth) += 1;
+    $opts = {} if !defined $opts;
 
-    if(@_ != 2) {
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
+
+    if(@_ < 2) {
         LOGCROAK("ask() called with wrong # of args");
     }
 
-    print STDERR "$prompt [$default]> "
-        or die "Couldn't write STDERR: ($!)";
-
-    my $value = <STDIN>;
-    chomp $value;
-
+    my $value = user_input("$prompt [$default]> ", $opts);
     $value = $default if $value eq "";
 
     return $value;
 }
 
+##################################################
+sub user_input {
+##################################################
+    my ($prompt, $opts) = @_;    
+
+    $opts = {} if !defined $opts;
+
+    my $fh = *STDIN;
+    if( $opts->{ tty } ) {
+        open $fh, "<", '/dev/tty' or 
+            die "Cannot open /dev/tty ($!)";
+    }
+
+    print STDERR $prompt
+        or die "Couldn't write STDERR: ($!)";
+
+    my $input = <$fh>;
+    chomp $input if defined $input;
+
+    return $input;
+}
+
 =pod
 
 =item C<mkd($dir)>
@@ -456,7 +498,8 @@ Create a directory of arbitrary depth, just like C<File::Path::mkpath>.
 sub mkd {
 ###############################################
 
-    local($Log::Log4perl::caller_depth) += 1;
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     _confirm "mkd @_" or return 1;
 
@@ -479,7 +522,8 @@ in the shell.
 sub rmf {
 ###############################################
 
-    local($Log::Log4perl::caller_depth) += 1;
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     _confirm "rmf $_[0]" or return 1;
 
@@ -510,7 +554,9 @@ set the stack_update parameter to a false value:
 sub cd {
 ###############################################
 
-    local($Log::Log4perl::caller_depth) += 1;
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
+
     INFO "cd $_[0]";
 
     my $opts = { stack_update => 1 };
@@ -532,18 +578,32 @@ sub cd {
 
 =item C<cdback()>
 
-chdir back to the last directory before a previous C<cd>.
+chdir back to the last directory before a previous C<cd>. If the
+option C<reset> is set, it goes all the way back to the beginning of the
+directory stack, i.e. no matter how many cd() calls were made in between,
+it'll go back to the original directory:
+
+      # go all the way back
+    cdback( { reset => 1 } );
 
 =cut
 
 ###############################################
 sub cdback {
 ###############################################
+    my( $opts ) = @_;
 
-    local($Log::Log4perl::caller_depth) += 1;
+    $opts = {} if !defined $opts;
+
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     LOGCROAK("cd stack empty") unless @DIR_STACK;
 
+    if( $opts->{ reset } ) {
+        @DIR_STACK = ( $DIR_STACK[0] );
+    }
+
     my $old_dir = pop @DIR_STACK;
 
     LOGCROAK("Directory stack empty")
@@ -565,7 +625,8 @@ Call C<make> in the shell.
 sub make {
 ###############################################
 
-    local($Log::Log4perl::caller_depth) += 1;
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     _confirm "make @_" or return 1;
 
@@ -597,7 +658,8 @@ sub archive_sniff {
 #######################################
     my($name) = @_;
 
-    local($Log::Log4perl::caller_depth) += 1;
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     DEBUG "Sniffing archive '$name'";
 
@@ -654,6 +716,11 @@ Example:
 
 Works with one or more file names.
 
+If the files are known to contain UTF-8 encoded data, and you want it
+to be read/written as a Unicode strings, use the C<utf8> option:
+
+    pie(sub { s/foo/bar/g; $_; }, "test.dat", { utf8 => 1 });
+
 =cut
 
 ###############################################
@@ -661,7 +728,15 @@ sub pie {
 ###############################################
     my($coderef, @files) = @_;
 
-    local($Log::Log4perl::caller_depth) += 1;
+    my $options = {};
+
+    if(defined $files[-1] and
+       ref $files[-1] eq "HASH") {
+       $options = pop @files;
+    }
+
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     for my $file (@files) {
 
@@ -671,12 +746,17 @@ sub pie {
 
         open FILE, "<$file" or 
             LOGCROAK("Cannot open $file ($!)");
+
+        if( $options->{utf8} ) {
+            binmode FILE, ":utf8";
+        }
+
         while(<FILE>) {
             $out .= $coderef->($_);
         }
         close FILE;
 
-        blurt($out, $file);
+        blurt($out, $file, $options);
     }
 }
 
@@ -694,6 +774,11 @@ Example:
 
 Works with one or more file names.
 
+If the files are known to contain UTF-8 encoded data, and you want it
+to be read into Unicode strings, use the C<utf8> option:
+
+    plough(sub { print if /foobar/ }, "test.dat", { utf8 => 1 });
+
 =cut
 
 ###############################################
@@ -701,7 +786,15 @@ sub plough {
 ###############################################
     my($coderef, @files) = @_;
 
-    local($Log::Log4perl::caller_depth) += 1;
+    my $options = {};
+
+    if(defined $files[-1] and
+        ref $files[-1] eq "HASH") {
+        $options = pop @files;
+    }
+
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     for my $file (@files) {
 
@@ -711,6 +804,11 @@ sub plough {
 
         open FILE, "<$file" or 
             LOGCROAK("Cannot open $file ($!)");
+
+        if( $options->{utf8} ) {
+            binmode FILE, ":utf8";
+        }
+
         while(<FILE>) {
             $coderef->($_);
         }
@@ -720,20 +818,28 @@ sub plough {
 
 =pod
 
-=item C<my $data = slurp($file)>
+=item C<my $data = slurp($file, $options)>
 
 Slurps in the file and returns a scalar with the file's content. If
 called without argument, data is slurped from STDIN or from any files
 provided on the command line (like E<lt>E<gt> operates).
 
+If the file is known to contain UTF-8 encoded data and you want to
+read it in as a Unicode string, use the C<utf8> option:
+
+    my $unicode_string = slurp( $file, {utf8 => 1} );
+
 =cut
 
 ###############################################
 sub slurp {
 ###############################################
-    my($file) = @_;
+    my($file, $options) = @_;
 
-    local($Log::Log4perl::caller_depth) += 1;
+    $options = {} unless defined $options;
+
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     my $from_file = defined($file);
 
@@ -745,6 +851,10 @@ sub slurp {
         INFO "Slurping data from $file";
         open FILE, "<$file" or 
             LOGCROAK("Cannot open $file ($!)");
+        binmode FILE; # Win32 wants that
+        if( exists $options->{utf8} ) {
+            binmode FILE, ":utf8";
+        }
         $data = <FILE>;
         close FILE;
         DEBUG "Read ", snip($data, $DATA_SNIPPED_LEN), " from $file";
@@ -765,23 +875,42 @@ Opens a new file, prints the data in C<$data> to it and closes the file.
 If C<$append> is set to a true value, data will be appended to the
 file. Default is false, existing files will be overwritten.
 
+If the string is a Unicode string, use the C<utf8> option:
+
+    blurt( $unicode_string, $file, {utf8 => 1} );
+
 =cut
 
 ###############################################
 sub blurt {
 ###############################################
-    my($data, $file, $append) = @_;
+    my($data, $file, $options) = @_;
 
-    local($Log::Log4perl::caller_depth) += 1;
+      # legacy signature
+    if(defined $options and ref $options eq "") {
+        $options = { append => 1 };
+    }
 
-    $append = 0 unless defined $append;
+    $options = {} unless defined $options;
 
-    _confirm(($append ? "Appending" : "Writing") . " " .
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
+
+    $options->{append} = 0 unless defined $options->{append};
+
+    _confirm(($options->{append} ? "Appending" : "Writing") . " " .
          length($data) . " bytes to $file") or return 1;
 
-    open FILE, ">" . ($append ? ">" : "") . $file 
+    open FILE, ">" . ($options->{append} ? ">" : "") . $file 
         or 
         LOGCROAK("Cannot open $file for writing ($!)");
+
+    binmode FILE; # Win32 wants that
+
+    if( $options->{utf8} ) {
+        binmode FILE, ":utf8";
+    }
+
     print FILE $data
         or 
         LOGCROAK("Cannot write to $file ($!)");        
@@ -794,7 +923,7 @@ sub blurt {
 
 =pod
 
-=item C<blurt_atomic($data, $file)>
+=item C<blurt_atomic($data, $file, $options)>
 
 Write the data in $data to a file $file, guaranteeing that the operation
 will either complete fully or not at all. This is accomplished by first
@@ -802,19 +931,27 @@ writing to a temporary file which is then rename()ed to the target file.
 
 Unlike in C<blurt>, there is no C<$append> mode in C<blurt_atomic>.
 
+If the string is a Unicode string, use the C<utf8> option:
+
+    blurt_atomic( $unicode_string, $file, {utf8 => 1} );
+
 =cut
 
 ###############################################
 sub blurt_atomic {
 ###############################################
-    my($data, $file) = @_;
+    my($data, $file, $options) = @_;
 
     _confirm("Writing atomically " .
          length($data) . " bytes to $file") or return 1;
 
+    $options = {} unless defined $options;
+
     my($fh, $tmpname) = tempfile(DIR => dirname($file));
 
-    blurt($data, $tmpname);
+    blurt($data, $tmpname, $options);
+
+    close $fh;
 
     rename $tmpname, $file or
         LOGDIE "Can't rename $tmpname to $file";
@@ -863,6 +1000,22 @@ wrapping all args so that shell variables are interpolated properly:
 
     "ls" "/tmp/$VAR" 2>/tmp/sometempfile |
 
+Another option is "utf8" which runs the command in a terminal set to 
+UTF8.
+
+Error handling: By default, tap() won't raise an error if the command's
+return code is nonzero, indicating an error reported by the shell. If 
+bailing out on errors is requested to avoid return code checking by
+the script, use the raise_error option:
+
+    tap({raise_error => 1}, "ls", "doesn't exist");
+
+In DEBUG mode, C<tap> logs the entire stdout/stderr output, which
+can get too verbose at times. To limit the number of bytes logged, use
+the C<stdout_limit> and C<stderr_limit> options
+
+    tap({stdout_limit => 10}, "echo", "123456789101112");
+
 =cut
 
 ###############################################
@@ -870,7 +1023,15 @@ sub tap {
 ###############################################
     my(@args) = @_;
 
-    local($Log::Log4perl::caller_depth) += 1;
+    my $options = {};
+
+    if(defined $args[-1] and
+       ref $args[-1] eq "HASH") {
+       $options = pop @args;
+    }
+
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     _confirm "tapping @args" or return 1;
 
@@ -899,12 +1060,29 @@ sub tap {
 
     open PIPE, $cmd or 
         LOGCROAK("open $cmd | failed ($!)");
+        
+    if( $options->{utf8} ) {
+        binmode PIPE, ":utf8";
+    }
+
     my $stdout = join '', <PIPE>;
     close PIPE;
 
     my $exit_code = $?;
 
-    my $stderr = slurp($tmpfile);
+    my $stderr = slurp($tmpfile, $options);
+
+    if( $opts->{ stderr_limit } ) {
+        $stderr = snip( $stderr, $opts->{ stderr_limit } );
+    }
+
+    if($exit_code != 0 and $opts->{raise_error}) {
+        LOGCROAK("tap $cmd | failed ($stderr)");
+    }
+
+    if( $opts->{ stdout_limit } ) {
+        $stdout = snip( $stdout, $opts->{ stdout_limit } );
+    }
 
     DEBUG "tap $cmd results: rc=$exit_code stderr=[$stderr] stdout=[$stdout]";
 
@@ -1078,7 +1256,8 @@ sub perm_cp {
     # Lifted from Ben Okopnik's
     # http://www.linuxgazette.com/issue87/misc/tips/cpmod.pl.txt
 
-    local($Log::Log4perl::caller_depth) += 1;
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     _confirm "perm_cp @_" or return 1;
 
@@ -1090,6 +1269,51 @@ sub perm_cp {
 
 =pod
 
+=item C<owner_cp($src, $dst, ...)>
+
+Read the C<$src> file/directory's owner uid and group gid and apply
+it to $dst.
+
+For example: copy uid/gid of the containing directory to a file
+therein:
+
+    use File::Basename;
+
+    owner_cp( dirname($file), $file );
+
+Usually requires root privileges, just like chown does.
+
+=cut
+
+######################################
+sub owner_cp {
+######################################
+    my($src, @dst) = @_;
+
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
+
+    _confirm "owner_cp @_" or return 1;
+
+    LOGCROAK("usage: owner_cp src dst ...") if @_ < 2;
+
+    my($uid, $gid) = (stat($src))[4,5];
+
+    if(!defined $uid or !defined $gid ) {
+        LOGCROAK("stat of $src failed: $!");
+        return undef;
+    }
+
+    if(!chown $uid, $gid, @dst ) {
+        LOGCROAK("chown of ", join(" ", @dst), " failed: $!");
+        return undef;
+    }
+
+    return 1;
+}
+
+=pod
+
 =item C<$perms = perm_get($filename)>
 
 Read the C<$filename>'s user permissions and owner/group. 
@@ -1103,7 +1327,8 @@ sub perm_get {
 ######################################
     my($filename) = @_;
 
-    local($Log::Log4perl::caller_depth) += 1;
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     my @stats = (stat $filename)[2,4,5] or
         
@@ -1129,7 +1354,8 @@ sub perm_set {
 ######################################
     my($filename, $perms) = @_;
 
-    local($Log::Log4perl::caller_depth) += 1;
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     _confirm "perm_set $filename (@$perms)" or return 1;
 
@@ -1156,7 +1382,8 @@ sub sysrun {
 ######################################
     my(@cmds) = @_;
 
-    local($Log::Log4perl::caller_depth) += 1;
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     _confirm "sysrun: @cmds" or return 1;
 
@@ -1182,7 +1409,8 @@ sub hammer {
 
     require Expect;
 
-    local($Log::Log4perl::caller_depth) += 1;
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
         _confirm "Hammer: @cmds" or return 1;
 
@@ -1230,7 +1458,8 @@ sub sudo_me {
 ######################################
     my($argv) = @_;
 
-    local($Log::Log4perl::caller_depth) += 1;
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     _confirm "sudo_me" or return 1;
 
@@ -1262,13 +1491,9 @@ sub bin_find {
 ######################################
     my($exe) = @_;
 
-    for my $path (split /:/, $ENV{PATH}) {
-        my $full = File::Spec->catfile($path, $exe);
-
-        return $full if -x $full;
-    }
-
-    return undef;
+      # File::Which returns a list in list context, we just want the first
+      # match.
+    return scalar File::Which::which( $exe );
 }
 
 =pod
@@ -1286,9 +1511,12 @@ This can be used to capture a file system structure.
 ######################################
 sub fs_read_open {
 ######################################
-    my($dir) = @_;
+    my($dir, $options) = @_;
 
-    local($Log::Log4perl::caller_depth) += 1;
+    $options = {} unless defined $options;
+
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     my $find = bin_find("find");
     LOGCROAK("Cannot find 'find'") unless defined $find;
@@ -1304,6 +1532,8 @@ sub fs_read_open {
     open my $in, "$cmd |" or 
         LOGCROAK("Cannot open $cmd");
 
+    binmode $in, ":utf8" if $options->{utf8};
+
     cdback;
 
     return $in;
@@ -1325,9 +1555,12 @@ with I<fs_read_open>.
 ######################################
 sub fs_write_open {
 ######################################
-    my($dir) = @_;
+    my($dir, $options) = @_;
 
-    local($Log::Log4perl::caller_depth) += 1;
+    $options = {} unless defined $options;
+
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     my $cpio = bin_find("cpio");
     LOGCROAK("Cannot find 'cpio'") unless defined $cpio;
@@ -1342,6 +1575,8 @@ sub fs_write_open {
     open my $out, "| $cmd" or 
         LOGCROAK("Cannot open $cmd");
 
+    binmode $out, ":utf8" if $options->{utf8};
+
     cdback;
 
     return $out;
@@ -1361,7 +1596,8 @@ sub pipe_copy {
 ######################################
     my($in, $out, $bufsize) = @_;
 
-    local($Log::Log4perl::caller_depth) += 1;
+    local $Log::Log4perl::caller_depth =
+          $Log::Log4perl::caller_depth + 1;
 
     $bufsize ||= 4096;
     my $bytes = 0;
@@ -1387,8 +1623,7 @@ sub pipe_copy {
 Format the data string in C<$data> so that it's only (roughly) $maxlen
 characters long and only contains printable characters.
 
-If C<$data> contains unprintable character's they are replaced by 
-"." (the dot). If C<$data> is longer than C<$maxlen>, it will be
+If C<$data> is longer than C<$maxlen>, it will be
 formatted like
 
     (22)[abcdef[snip=11]stuvw]
@@ -1396,6 +1631,12 @@ formatted like
 indicating the length of the original string, the beginning, the
 end, and the number of 'snipped' characters.
 
+If C<$data> is shorter than $maxlen, it will be returned unmodified 
+(except for unprintable characters replaced, see below).
+
+If C<$data> contains unprintable character's they are replaced by 
+"." (the dot).
+
 =cut
 
 ###########################################
@@ -1404,7 +1645,7 @@ sub snip {
     my($data, $maxlen) = @_;
 
     if(length $data <= $maxlen) {
-        return lenformat($data);
+        return printable($data);
     }
 
     $maxlen = 12 if $maxlen < 12;
@@ -1431,7 +1672,7 @@ sub printable {
 ###########################################
     my($data) = @_;
 
-    $data =~ s/[^ \w.;!?@#$%^&*()+\\|~`'-,><[\]{}="]/./g;
+    $data =~ s/[^ \w.;!?@#$%^&*()+\\|~`',><[\]{}="-]/./g;
     return $data;
 }
 
@@ -1545,6 +1786,62 @@ sub def_or($$) {
     }
 }
 
+=item C<is_utf8_data($data)>
+
+Check if the given string has the utf8 flag turned on. Works just like 
+Encode.pm's is_utf8() function, except that it silently returns a 
+false if Encode isn't available, for example when an ancient perl 
+without proper utf8 support is used.
+
+=cut
+
+###############################################
+sub is_utf8_data {
+###############################################
+    my($data) = @_;
+
+    if( !utf8_available() ) {
+        return 0;
+    }
+
+    return Encode::is_utf8( $data );
+}
+
+=item C<utf8_check($data)>
+
+Check if we're using a perl with proper utf8 support, by verifying the
+Encode.pm module is available for loading.
+
+=cut
+
+###############################################
+sub utf8_available {
+###############################################
+
+    eval "use Encode";
+
+    if($@) {
+        return 0;
+    }
+
+    return 1;
+}
+
+=item C<home_dir()>
+
+Return the path to the home directory of the current user.
+
+=cut
+
+###############################################
+sub home_dir {
+###############################################
+
+    my( $home ) = glob "~";
+
+    return $home;
+}
+
 =pod
 
 =back
